/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


/*****************************************************************************  
 * This sketch draws the Japanese flag, changes the position of flag corner
 * using different animation functions and changes the saturation of the dot's
 * color
 *****************************************************************************/

float dotSize;  // size of the flag's dot in pixels
float dotHue;   // color (hue in HSB) of the flag's dot
float dotSat;   // saturation of the dot's color


/* -------------------------------------------------------------------
 * The setup function is run once at the beginning of the application.
 * Here you setup any initial values of variables, settings, etc.
 * ------------------------------------------------------------------- */
void setup() {
  
  // the size function MUST be called at the beginning of the setup function
  // to determine the size of the application window
  size(1280, 720);
  // fullScreen();  // comment the line above and uncomment this line to run the application in full screen
  
  // The dot's size is set to a forth of the window's height
  dotSize = width/4;
  
  // Change color mode from RGB to HSB with Hue in range [0,360), Saturation in [0, 100), Brightness in [0, 100)
  colorMode(HSB, 360, 100, 100);  
  
  // Set hue to 0°, i.e. red
  dotHue = 0;
  dotSat = 100;
}


/* -------------------------------------------------------------------
 * The draw function is run repeatedly until the application quits.
 * Here you code what is drawn at each frame of the running application. 
 * ------------------------------------------------------------------- */
void draw() {
  background(0, 0, 0);
  
  // animation often uses a representation of the application's time to 
  // change values. The millis function returns the milliseconds of the 
  // application's time running as an integer number. 
  int ms = millis() % 4000;  // The variable ms will hold values in the range of 
                             // [0, 3999] due to the modulo (% 4000) operator 
  float animationTime = 0;   // animation time will hold values in the range of [0.0, 1.0)
                             // representing the time in the current animation cycle
  
  // in the first 2000ms, let animation time run from 0.0 to 1.0
  // then in the second 2000ms (2000 to 4000) let animation time run from 
  // 1.0 to 0.0  to obtain a back-and-forth motion
  if (ms < 2000) { 
    animationTime = ms/2000.0;
  }
  else {
    animationTime = (4000 - ms)/2000.0; 
  }
  
  // compute flag corner positions
  // see the definition of the functions at the end of the sketch
  // and the tab AnimationFunctions
  PVector p1 = upperLeftCorner(animationTime);  // compute position of upper left corner with linear motion, i.e. equal velocity
  PVector p2 = upperRightCorner(animationTime); // compute position of upper right corner with quadratic motion, i.e. increasing velocity
  PVector p3 = lowerRightCorner(animationTime); // compute position of lower right corner with inverse quadratic motion, i.e. decreasing velocity
  PVector p4 = lowerLeftCorner(animationTime);  // compute position of lower left corner with quadratic motion in and out of the animation, 
                                                //      i.e. increasing velocity at the beginning and decreasing at the end
  
  dotSat = linear(animationTime, 100, 0);   // animate dot saturation with linear change
  
  // draw flag background with a quad that has arbitrary corners
  noStroke();
  fill(0, 0, 100);
  quad(p1.x, p1.y, p2.x, p2.y, p3.x, p3.y, p4.x, p4.y);
  
  // draw dot
  drawDot();
  
  
  // draw spacing points of the different animations
  fill(0, 100, 100);
  for (int i = 0; i < 20; i++) {
    float s = i / 19.0;
    
    if (s > animationTime)
      fill(0, 0, 0);
    else
      fill(0, 0, 100);
    
    p1 = upperLeftCorner(s);
    p2 = upperRightCorner(s);
    p3 = lowerRightCorner(s);
    p4 = lowerLeftCorner(s);
    ellipse(p1.x, p1.y, 3, 3);
    ellipse(p2.x, p2.y, 3, 3);
    ellipse(p3.x, p3.y, 5, 5);
    ellipse(p4.x, p4.y, 4, 4);
  }
  
}



/* -------------------------------------------------------------------
 * The drawFunction draws a dot at position (x, y)
 * ------------------------------------------------------------------- */
void drawDot() {
  
  // draw dot: First set dot color, then draw an ellipse with identical size along x- and y-axis
  fill(dotHue, dotSat, 100);      // <-- colors are now given by HSB
  ellipse(width/2, height/2, dotSize, dotSize);
}

PVector upperLeftCorner(float animationTime) {
  return linear(animationTime, new PVector(0, 0), new PVector(width/3.0, height/3.0));
}

PVector upperRightCorner(float animationTime) {
  return nonlinear(animationTime, 2, new PVector(width, 0), new PVector(width - width/3.0, height/3.0));
}

PVector lowerRightCorner(float animationTime) {
  return nonlinearInverse(animationTime, 2, new PVector(width, height), new PVector(width - width/3.0, height - height/3.0));
}

PVector lowerLeftCorner(float animationTime) {
  return nonlinearInOut(animationTime, 2, new PVector(0, height), new PVector(width/3.0, height - height/3.0));
}