/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


/*****************************************************************************  
 * This sketch draws the Japanese flag with animation from sound input
 *****************************************************************************/
  
import processing.sound.*;  // import sound library

Amplitude amp;              // reference to audio amplitude analyzer object
AudioIn in;                 // reference to audio input, e.g. from notebook microphone

float dotSize;              // size of the flag's dot in pixels
float rotationAngle = 90;   // angle of rotation in degrees, i.e. direction of where the dot moves according to current amplitude
                            //      the rotation is increased with every frame to achieve a circle-like motion


/* -------------------------------------------------------------------
 * The setup function is run once at the beginning of the application.
 * Here you setup any initial values of variables, settings, etc.
 * ------------------------------------------------------------------- */
void setup() {
  
  // the size function MUST be called at the beginning of the setup function
  // to determine the size of the application window
  size(600, 400);
  //fullScreen(P3D, 2);  

  amp = new Amplitude(this);   // create new amplitude analyzer object
  in = new AudioIn(this, 0);   // create new audio input
  in.start();                  // start audio input
  amp.input(in);               // connect amplitude analyzer to input
  
  
  // The dot's size is set to half of the window's height
  dotSize = height/2;
  
  // rectangles are now specified by their width, height and center instead of upper left corner 
  rectMode(CENTER);
  
  // init first frame with white background
  background(255);
}


/* -------------------------------------------------------------------
 * The draw function is run repeatedly until the application quits.
 * Here you code what is drawn at each frame of the running application. 
 * ------------------------------------------------------------------- */
void draw() {
  
  noStroke();
  
  // paint the white flag background with transparency.
  // this leaves the latest frames still visible and fading out over time
  // to create trails/echos of the past graphics
  fill(255, 20);                          // color is (r=255, g=255, b=255, alpha=20)
  rect(width/2, height/2, width, height); 
  
  float amplitude = amp.analyze(); // obtain current amplitude
  
  
  // instead of drawing graphics at a certain locations, you can also move the application's 
  // coordinate system. The understanding of this concept becomes quite handy once you need 
  // to move lots of graphics depending on their locations.
  // Have a look at the drawDot function: the red dot is drawn always with y-position=0 and
  // x-position depending on the amplitude. The previous function calls of translate and rotate
  // translate the coordinate system to the window's center and rotate the coordinate system
  // according to the rotation angle.
  translate(width/2, height/2);
  rotate(radians(rotationAngle));   // rotate function takes radians, i.e. [0, 2*PI). the randians
                                    // function converts degrees [0, 360) to radians

  // draw the dot
  drawDot(amplitude);
  
  // increase rotation angle
  rotationAngle += 4;
}



/* -------------------------------------------------------------------
 * The drawDot function draws the dot
 * ------------------------------------------------------------------- */
void drawDot(float amplitude) {
  
  // draw a white circle
  stroke(255, 180);
  noFill();
  ellipse(0, 0, dotSize*(1.0-amplitude), dotSize*(1.0-amplitude));
  
  // draw the red dot
  noStroke();
  fill(255, 0, 0);
  ellipse(amplitude*height, 0, dotSize*(1.0-amplitude), dotSize*(1.0-amplitude));
  
}